from abc import ABC
from datetime import datetime
from dataclasses import field
from uuid import UUID

from pydantic import BaseModel, Field
from pydantic.dataclasses import dataclass

from inferscope.models.user import UserInfo


class InteractiveBaseModel(BaseModel):
    uid: UUID | None = Field(init=False, default=None)
    created_by: UserInfo | None = Field(init=False, default=None)
    created_ts: datetime | None = Field(init=False, default=None)
    updated_ts: datetime | None = Field(init=False, default=None)
    access_ts: datetime | None = Field(init=False, default=None)

    @classmethod
    def entity_name(cls) -> str:
        pass

    @classmethod
    def entity_url(cls, entity_id: str | UUID) -> str:
        return f"{cls.entity_name()}/{entity_id}"

    @property
    def entity_create_url(self) -> str:
        return self.entity_name()

    def entity_delete_url(self) -> str:
        return self.entity_url(self.uid)

    @property
    def entity_search_url(self) -> str:
        return f"{self.entity_name()}s/"

    def make_create_request(self):
        return self.CreateRequest.model_validate(self)



from enum import StrEnum
from typing import Literal, List

from pydantic import BaseModel, Field

from inferscope.models.data_description import DataDescription


class ArtifactType(StrEnum):
    EXTERNAL_LINK = "external_link"


class BaseArtifact(BaseModel):
    path: str = Field(min_length=1, max_length=120)
    type: ArtifactType
    data_description: DataDescription | None = None


class ExternalLinkArtifact(BaseArtifact):
    type: Literal[ArtifactType.EXTERNAL_LINK] = ArtifactType.EXTERNAL_LINK
    uri: str


class ArtifactInstance(BaseModel):
    name: str
    version: str | None = None
    description: str | None = None


class ArtifactPack(BaseModel):
    artifacts: List[ExternalLinkArtifact] | None = None



from datetime import datetime
from enum import StrEnum
from uuid import UUID

from pydantic import ConfigDict, Field

from inferscope.models.artifact import ArtifactPack
from inferscope.models.dataset import DatasetInfo
from inferscope.models.find_filter import BaseFindFilter
from inferscope.models.metric import Metrics
from inferscope.models.model import ModelInfo
from inferscope.models.interactive_base import InteractiveBaseModel


class RunStatus(StrEnum):
    DRAFT = "draft"
    RUNNING = "running"
    FAILED = "failed"
    DONE = "done"


class RunUpdateRequest(ArtifactPack):
    name: str | None = Field(min_length=1, max_length=255, default=None)
    description: str | None = None
    metrics: Metrics | None = None
    status: RunStatus | None = None


class Run(InteractiveBaseModel, ArtifactPack):
    class CreateRequest(ArtifactPack):
        owner_project: UUID | None = None

        name: str | None = Field(min_length=1, max_length=255, default=None)
        description: str | None = None
        dataset: DatasetInfo | None = None
        metrics: Metrics | None = None
        model: ModelInfo | None = None
        status: RunStatus = RunStatus.DONE
        tags: list[str] | None = None

        override_datetime: datetime | None = None

    model_config = ConfigDict(extra="allow")

    @classmethod
    def entity_name(cls) -> str:
        return "run"

    access_ts: datetime | None = None
    parent_project_uid: UUID | None = None
    parent_experiment_uid: UUID | None = None

    name: str | None = Field(min_length=1, max_length=255, default=None)

    dataset: DatasetInfo | None = None
    description: str | None = None
    metrics: Metrics | None = None
    model: ModelInfo | None = None
    status: RunStatus = RunStatus.DONE
    tags: list[str] | None = None


class RunFindFilter(BaseFindFilter):
    project_id: str | None = None
    experiment_id: str | None = None



from enum import StrEnum

from pydantic import BaseModel, Field


class MetricBestValue(StrEnum):
    MIN = "min"
    MAX = "max"


class MetricType(StrEnum):
    SCALAR = "scalar"
    SERIES = "series"
    TIME_SERIES = "time_series"


class MetricKeyParameters(BaseModel):
    name: str = Field(min_length=1, max_length=120)
    slice: str | None = None
    type: MetricType = MetricType.SCALAR
    best_value: MetricBestValue | None = None


class Metric(MetricKeyParameters):
    value: float | int | list[float] | list[int]
    std: float | None = None


class Metrics(BaseModel):
    metrics: list[Metric]



from enum import StrEnum
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field

from inferscope.models.interactive_base import InteractiveBaseModel
from inferscope.models.find_filter import BaseFindFilter


class Experiment(InteractiveBaseModel):
    class Status(StrEnum):
        DRAFT = "draft"
        ACTIVE = "active"
        FINISHED = "finished"
        FAILED = "failed"
        ARCHIVED = "archived"

    class CreateRequest(BaseModel):
        name: str = Field(min_length=1, max_length=255)
        parent_project: UUID | None = None
        description: str | None = None
        tags: list[str] | None = None

    @classmethod
    def entity_name(cls) -> str:
        return "experiment"

    model_config = ConfigDict(extra="allow")
    parent_project: UUID
    status: Status
    name: str = Field(min_length=1, max_length=255)
    description: str | None = None
    tags: list[str] | None = None


class ExperimentFindFilter(BaseFindFilter):
    project_list: list[UUID] | None = None



from datetime import datetime
from enum import StrEnum
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field

from inferscope.models.find_filter import BaseFindFilter
from inferscope.models.user import UserInfo

from inferscope.models.interactive_base import InteractiveBaseModel


class Project(InteractiveBaseModel):
    class Status(StrEnum):
        ACTIVE = "active"
        ARCHIVED = "archived"

    class CreateRequest(BaseModel):
        name: str = Field(min_length=1, max_length=255)
        description: str | None = None
        tags: list[str] | None = None

    @classmethod
    def entity_name(cls) -> str:
        return "project"

    model_config = ConfigDict(extra="allow")
    status: Status
    owner: UserInfo
    members: list[UserInfo]
    name: str = Field(min_length=1, max_length=255)
    description: str | None = None
    tags: list[str] | None = None


class ProjectFindFilter(BaseFindFilter):
    pass



from datetime import datetime

from pydantic import BaseModel


class BaseFindFilter(BaseModel):
    limit: int = 100
    offset: int = 0
    created_lt: datetime | None = None
    created_gte: datetime | None = None
    tags: list[str] | None = None



import os
from setuptools import setup, find_packages

PREFIX = "inferscope"


def get_version():
    setup_py_dir = os.path.dirname(__file__)
    version_module_path = os.path.join(setup_py_dir, "src", "inferscope", "__version__.py")

    about = {}

    with open(version_module_path) as f:
        exec(f.read(), about)  # noqa

    return about["__version__"]


def setup_package():
    setup(
        name="inferscope",
        version=get_version(),
        author="kizill",
        author_email="staskirillov@gmail.com",
        packages=find_packages("src"),
        package_dir={"": "src"},
        description="Python-package for inferscope mlops service",
        include_package_data=True,
        install_requires=[
            "attr",
            "attrs",
            "httpx[http2]",
            "pydantic>2.0",
        ],
        python_requires=">=3.7.0",
        keywords=["ml", "mlops"],
    )


if __name__ == "__main__":
    setup_package()



from enum import StrEnum, auto

from pydantic import BaseModel, ConfigDict, Extra


class DataType(StrEnum):
    Integer = auto()
    Float = auto()
    String = auto()
    Boolean = auto()
    Date = auto()

    URL = auto()  # string variant

    Bytes = auto()  # for inline image/audio

    Array = auto()
    Dictionary = auto()


class SemanticType(StrEnum):
    Prediction = auto()
    Metric = auto()

    Timestamp = auto()  # semantic for integer field

    Image = auto()
    Audio = auto()
    Video = auto()
    Code = auto()


class JoinType(StrEnum):
    Inner = auto()


class ColumnInformation(BaseModel):
    name: str
    title: str | None = None
    tooltip: str | None = None
    type: DataType
    group_id: str | None = None
    show: bool = True
    semantic: SemanticType | None = None
    join_type: JoinType | None = None

    model_config = ConfigDict(extra=Extra.forbid)


class DataFormatType(StrEnum):
    JSON = auto()
    DSV = auto()
    Binary = auto()
    Pickle = auto()  # for insecure serialized formats


class DataFormat(BaseModel):
    data_format: DataFormatType
    dsv_delimiter: str | None = None  # for DSV

    model_config = ConfigDict(extra=Extra.forbid)


class DataDescription(BaseModel):
    columns: list[ColumnInformation]
    data_format: DataFormat

    model_config = ConfigDict(extra=Extra.forbid)
